# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("projectedfslib.dll")
prototypes = \
    {
        # 
        'PrjStartVirtualizing': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"StartDirectoryEnumerationCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0), "EndDirectoryEnumerationCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId"]), offset=0), "GetDirectoryEnumerationCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "enumerationId", "searchExpression", "dirEntryBufferHandle"]), offset=0), "GetPlaceholderInfoCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0), "GetFileDataCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "byteOffset", "length"]), offset=0), "QueryFileNameCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData"]), offset=0), "NotificationCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0), SimTypeChar(label="Byte"), SimTypeInt(signed=False, label="PRJ_NOTIFICATION"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimUnion({"PostCreate": SimStruct({"NotificationMask": SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")}, name="_PostCreate_e__Struct", pack=False, align=None), "FileRenamed": SimStruct({"NotificationMask": SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")}, name="_FileRenamed_e__Struct", pack=False, align=None), "FileDeletedOnHandleClose": SimStruct({"IsFileModified": SimTypeChar(label="Byte")}, name="_FileDeletedOnHandleClose_e__Struct", pack=False, align=None)}, name="<anon>", label="None"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["callbackData", "isDirectory", "notification", "destinationFileName", "operationParameters"]), offset=0), "CancelCommandCallback": SimTypePointer(SimTypeFunction([SimTypePointer(SimStruct({"Size": SimTypeInt(signed=False, label="UInt32"), "Flags": SimTypeInt(signed=False, label="PRJ_CALLBACK_DATA_FLAGS"), "NamespaceVirtualizationContext": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), "CommandId": SimTypeInt(signed=True, label="Int32"), "FileId": SimTypeBottom(label="Guid"), "DataStreamId": SimTypeBottom(label="Guid"), "FilePathName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "VersionInfo": SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), "TriggeringProcessId": SimTypeInt(signed=False, label="UInt32"), "TriggeringProcessImageFileName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "InstanceContext": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="PRJ_CALLBACK_DATA", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["callbackData"]), offset=0)}, name="PRJ_CALLBACKS", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypePointer(SimStruct({"Flags": SimTypeInt(signed=False, label="PRJ_STARTVIRTUALIZING_FLAGS"), "PoolThreadCount": SimTypeInt(signed=False, label="UInt32"), "ConcurrentThreadCount": SimTypeInt(signed=False, label="UInt32"), "NotificationMappings": SimTypePointer(SimStruct({"NotificationBitMask": SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES"), "NotificationRoot": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="PRJ_NOTIFICATION_MAPPING", pack=False, align=None), offset=0), "NotificationMappingsCount": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_STARTVIRTUALIZING_OPTIONS", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["virtualizationRootPath", "callbacks", "instanceContext", "options", "namespaceVirtualizationContext"]),
        # 
        'PrjStopVirtualizing': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["namespaceVirtualizationContext"]),
        # 
        'PrjClearNegativePathCache': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "totalEntryNumber"]),
        # 
        'PrjGetVirtualizationInstanceInfo': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"InstanceID": SimTypeBottom(label="Guid"), "WriteAlignment": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_VIRTUALIZATION_INSTANCE_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "virtualizationInstanceInfo"]),
        # 
        'PrjMarkDirectoryAsPlaceholder': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["rootPathName", "targetPathName", "versionInfo", "virtualizationInstanceID"]),
        # 
        'PrjWritePlaceholderInfo': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"FileBasicInfo": SimStruct({"IsDirectory": SimTypeChar(label="Byte"), "FileSize": SimTypeLongLong(signed=True, label="Int64"), "CreationTime": SimTypeBottom(label="LARGE_INTEGER"), "LastAccessTime": SimTypeBottom(label="LARGE_INTEGER"), "LastWriteTime": SimTypeBottom(label="LARGE_INTEGER"), "ChangeTime": SimTypeBottom(label="LARGE_INTEGER"), "FileAttributes": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_FILE_BASIC_INFO", pack=False, align=None), "EaInformation": SimStruct({"EaBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstEa": SimTypeInt(signed=False, label="UInt32")}, name="_EaInformation_e__Struct", pack=False, align=None), "SecurityInformation": SimStruct({"SecurityBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToSecurityDescriptor": SimTypeInt(signed=False, label="UInt32")}, name="_SecurityInformation_e__Struct", pack=False, align=None), "StreamsInformation": SimStruct({"StreamsInfoBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstStreamInfo": SimTypeInt(signed=False, label="UInt32")}, name="_StreamsInformation_e__Struct", pack=False, align=None), "VersionInfo": SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), "VariableData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PRJ_PLACEHOLDER_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "destinationFileName", "placeholderInfo", "placeholderInfoSize"]),
        # 
        'PrjWritePlaceholderInfo2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"FileBasicInfo": SimStruct({"IsDirectory": SimTypeChar(label="Byte"), "FileSize": SimTypeLongLong(signed=True, label="Int64"), "CreationTime": SimTypeBottom(label="LARGE_INTEGER"), "LastAccessTime": SimTypeBottom(label="LARGE_INTEGER"), "LastWriteTime": SimTypeBottom(label="LARGE_INTEGER"), "ChangeTime": SimTypeBottom(label="LARGE_INTEGER"), "FileAttributes": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_FILE_BASIC_INFO", pack=False, align=None), "EaInformation": SimStruct({"EaBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstEa": SimTypeInt(signed=False, label="UInt32")}, name="_EaInformation_e__Struct", pack=False, align=None), "SecurityInformation": SimStruct({"SecurityBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToSecurityDescriptor": SimTypeInt(signed=False, label="UInt32")}, name="_SecurityInformation_e__Struct", pack=False, align=None), "StreamsInformation": SimStruct({"StreamsInfoBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstStreamInfo": SimTypeInt(signed=False, label="UInt32")}, name="_StreamsInformation_e__Struct", pack=False, align=None), "VersionInfo": SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), "VariableData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PRJ_PLACEHOLDER_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"InfoType": SimTypeInt(signed=False, label="PRJ_EXT_INFO_TYPE"), "NextInfoOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"Symlink": SimStruct({"TargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="_Symlink_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="PRJ_EXTENDED_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "destinationFileName", "placeholderInfo", "placeholderInfoSize", "ExtendedInfo"]),
        # 
        'PrjUpdateFileIfNeeded': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"FileBasicInfo": SimStruct({"IsDirectory": SimTypeChar(label="Byte"), "FileSize": SimTypeLongLong(signed=True, label="Int64"), "CreationTime": SimTypeBottom(label="LARGE_INTEGER"), "LastAccessTime": SimTypeBottom(label="LARGE_INTEGER"), "LastWriteTime": SimTypeBottom(label="LARGE_INTEGER"), "ChangeTime": SimTypeBottom(label="LARGE_INTEGER"), "FileAttributes": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_FILE_BASIC_INFO", pack=False, align=None), "EaInformation": SimStruct({"EaBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstEa": SimTypeInt(signed=False, label="UInt32")}, name="_EaInformation_e__Struct", pack=False, align=None), "SecurityInformation": SimStruct({"SecurityBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToSecurityDescriptor": SimTypeInt(signed=False, label="UInt32")}, name="_SecurityInformation_e__Struct", pack=False, align=None), "StreamsInformation": SimStruct({"StreamsInfoBufferSize": SimTypeInt(signed=False, label="UInt32"), "OffsetToFirstStreamInfo": SimTypeInt(signed=False, label="UInt32")}, name="_StreamsInformation_e__Struct", pack=False, align=None), "VersionInfo": SimStruct({"ProviderID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128), "ContentID": SimTypeFixedSizeArray(SimTypeChar(label="Byte"), 128)}, name="PRJ_PLACEHOLDER_VERSION_INFO", pack=False, align=None), "VariableData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="PRJ_PLACEHOLDER_INFO", pack=False, align=None), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="PRJ_UPDATE_TYPES"), SimTypePointer(SimTypeInt(signed=False, label="PRJ_UPDATE_FAILURE_CAUSES"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "destinationFileName", "placeholderInfo", "placeholderInfoSize", "updateFlags", "failureReason"]),
        # 
        'PrjDeleteFile': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="PRJ_UPDATE_TYPES"), SimTypePointer(SimTypeInt(signed=False, label="PRJ_UPDATE_FAILURE_CAUSES"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "destinationFileName", "updateFlags", "failureReason"]),
        # 
        'PrjWriteFileData': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0), SimTypeLongLong(signed=False, label="UInt64"), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "dataStreamId", "buffer", "byteOffset", "length"]),
        # 
        'PrjGetOnDiskFileState': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="PRJ_FILE_STATE"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["destinationFileName", "fileState"]),
        # 
        'PrjAllocateAlignedBuffer': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt"), label="UIntPtr", offset=0)], SimTypePointer(SimTypeBottom(label="Void"), offset=0), arg_names=["namespaceVirtualizationContext", "size"]),
        # 
        'PrjFreeAlignedBuffer': SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["buffer"]),
        # 
        'PrjCompleteCommand': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=True, label="Int32"), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimStruct({"CommandType": SimTypeInt(signed=False, label="PRJ_COMPLETE_COMMAND_TYPE"), "Anonymous": SimUnion({"Notification": SimStruct({"NotificationMask": SimTypeInt(signed=False, label="PRJ_NOTIFY_TYPES")}, name="_Notification_e__Struct", pack=False, align=None), "Enumeration": SimStruct({"DirEntryBufferHandle": SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)}, name="_Enumeration_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="PRJ_COMPLETE_COMMAND_EXTENDED_PARAMETERS", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["namespaceVirtualizationContext", "commandId", "completionResult", "extendedParameters"]),
        # 
        'PrjFillDirEntryBuffer': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"IsDirectory": SimTypeChar(label="Byte"), "FileSize": SimTypeLongLong(signed=True, label="Int64"), "CreationTime": SimTypeBottom(label="LARGE_INTEGER"), "LastAccessTime": SimTypeBottom(label="LARGE_INTEGER"), "LastWriteTime": SimTypeBottom(label="LARGE_INTEGER"), "ChangeTime": SimTypeBottom(label="LARGE_INTEGER"), "FileAttributes": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_FILE_BASIC_INFO", pack=False, align=None), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["fileName", "fileBasicInfo", "dirEntryBufferHandle"]),
        # 
        'PrjFillDirEntryBuffer2': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimStruct({"IsDirectory": SimTypeChar(label="Byte"), "FileSize": SimTypeLongLong(signed=True, label="Int64"), "CreationTime": SimTypeBottom(label="LARGE_INTEGER"), "LastAccessTime": SimTypeBottom(label="LARGE_INTEGER"), "LastWriteTime": SimTypeBottom(label="LARGE_INTEGER"), "ChangeTime": SimTypeBottom(label="LARGE_INTEGER"), "FileAttributes": SimTypeInt(signed=False, label="UInt32")}, name="PRJ_FILE_BASIC_INFO", pack=False, align=None), offset=0), SimTypePointer(SimStruct({"InfoType": SimTypeInt(signed=False, label="PRJ_EXT_INFO_TYPE"), "NextInfoOffset": SimTypeInt(signed=False, label="UInt32"), "Anonymous": SimUnion({"Symlink": SimStruct({"TargetName": SimTypePointer(SimTypeChar(label="Char"), offset=0)}, name="_Symlink_e__Struct", pack=False, align=None)}, name="<anon>", label="None")}, name="PRJ_EXTENDED_INFO", pack=False, align=None), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["dirEntryBufferHandle", "fileName", "fileBasicInfo", "extendedInfo"]),
        # 
        'PrjFileNameMatch': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeChar(label="Byte"), arg_names=["fileNameToCheck", "pattern"]),
        # 
        'PrjFileNameCompare': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["fileName1", "fileName2"]),
        # 
        'PrjDoesNameContainWildCards': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeChar(label="Byte"), arg_names=["fileName"]),
    }

lib.set_prototypes(prototypes)



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("api-ms-win-core-featurestaging-l1-1-0.dll")
prototypes = \
    {
        # 
        'GetFeatureEnabledState': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="FEATURE_CHANGE_TIME")], SimTypeInt(signed=False, label="FEATURE_ENABLED_STATE"), arg_names=["featureId", "changeTime"]),
        # 
        'RecordFeatureUsage': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), offset=0)], SimTypeBottom(label="Void"), arg_names=["featureId", "kind", "addend", "originName"]),
        # 
        'RecordFeatureError': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"hr": SimTypeInt(signed=True, label="Int32"), "lineNumber": SimTypeShort(signed=False, label="UInt16"), "file": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "process": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "module": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "callerReturnAddressOffset": SimTypeInt(signed=False, label="UInt32"), "callerModule": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "message": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "originLineNumber": SimTypeShort(signed=False, label="UInt16"), "originFile": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "originModule": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "originCallerReturnAddressOffset": SimTypeInt(signed=False, label="UInt32"), "originCallerModule": SimTypePointer(SimTypeChar(label="Byte"), offset=0), "originName": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="FEATURE_ERROR", pack=False, align=None), offset=0)], SimTypeBottom(label="Void"), arg_names=["featureId", "error"]),
        # 
        'SubscribeFeatureStateChangeNotification': SimTypeFunction([SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["context"]), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["subscription", "callback", "context"]),
        # 
        'UnsubscribeFeatureStateChangeNotification': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeBottom(label="Void"), arg_names=["subscription"]),
    }

lib.set_prototypes(prototypes)



import angr

class wcscmp(angr.SimProcedure):
    #pylint:disable=arguments-differ
    def run(self, lpString1, lpString2):
        strcmp = angr.SIM_PROCEDURES['libc']['strcmp']
        return self.inline_call(strcmp, lpString1, lpString2, wchar=True).ret_expr

class wcscasecmp(angr.SimProcedure):
    #pylint:disable=arguments-differ
    def run(self, lpString1, lpString2):
        strcmp = angr.SIM_PROCEDURES['libc']['strcmp']
        return self.inline_call(strcmp, lpString1, lpString2, wchar=True, ignore_case=True).ret_expr



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("traffic.dll")
prototypes = \
    {
        # 
        'TcRegisterClient': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"ClNotifyHandler": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeBottom(label="Void"), arg_names=["ClRegCtx", "ClIfcCtx", "Event", "SubCode", "BufSize", "Buffer"]), offset=0), "ClAddFlowCompleteHandler": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0), "ClModifyFlowCompleteHandler": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0), "ClDeleteFlowCompleteHandler": SimTypePointer(SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeBottom(label="Void"), arg_names=["ClFlowCtx", "Status"]), offset=0)}, name="TCI_CLIENT_FUNC_LIST", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["TciVersion", "ClRegCtx", "ClientHandlerList", "pClientHandle"]),
        # 
        'TcEnumerateInterfaces': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"Length": SimTypeInt(signed=False, label="UInt32"), "pInterfaceName": SimTypePointer(SimTypeChar(label="Char"), offset=0), "pInterfaceID": SimTypePointer(SimTypeChar(label="Char"), offset=0), "AddressListDesc": SimStruct({"MediaType": SimTypeInt(signed=False, label="UInt32"), "AddressList": SimStruct({"AddressCount": SimTypeInt(signed=True, label="Int32"), "AddressType": SimTypeShort(signed=False, label="UInt16"), "Address": SimTypePointer(SimStruct({"AddressLength": SimTypeShort(signed=False, label="UInt16"), "AddressType": SimTypeShort(signed=False, label="UInt16"), "Address": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="NETWORK_ADDRESS", pack=False, align=None), offset=0)}, name="NETWORK_ADDRESS_LIST", pack=False, align=None)}, name="ADDRESS_LIST_DESCRIPTOR", pack=False, align=None)}, name="TC_IFC_DESCRIPTOR", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ClientHandle", "pBufferSize", "InterfaceBuffer"]),
        # 
        'TcOpenInterfaceA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInterfaceName", "ClientHandle", "ClIfcCtx", "pIfcHandle"]),
        # 
        'TcOpenInterfaceW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pInterfaceName", "ClientHandle", "ClIfcCtx", "pIfcHandle"]),
        # 
        'TcCloseInterface': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["IfcHandle"]),
        # 
        'TcQueryInterface': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeChar(label="Byte"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["IfcHandle", "pGuidParam", "NotifyChange", "pBufferSize", "Buffer"]),
        # 
        'TcSetInterface': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["IfcHandle", "pGuidParam", "BufferSize", "Buffer"]),
        # 
        'TcQueryFlowA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pFlowName", "pGuidParam", "pBufferSize", "Buffer"]),
        # 
        'TcQueryFlowW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pFlowName", "pGuidParam", "pBufferSize", "Buffer"]),
        # 
        'TcSetFlowA': SimTypeFunction([SimTypePointer(SimTypeChar(label="Byte"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pFlowName", "pGuidParam", "BufferSize", "Buffer"]),
        # 
        'TcSetFlowW': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeBottom(label="Void"), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["pFlowName", "pGuidParam", "BufferSize", "Buffer"]),
        # 
        'TcAddFlow': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"SendingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "ReceivingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "TcObjectsLength": SimTypeInt(signed=False, label="UInt32"), "TcObjects": SimTypePointer(SimStruct({"ObjectType": SimTypeInt(signed=False, label="UInt32"), "ObjectLength": SimTypeInt(signed=False, label="UInt32")}, name="QOS_OBJECT_HDR", pack=False, align=None), offset=0)}, name="TC_GEN_FLOW", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["IfcHandle", "ClFlowCtx", "Flags", "pGenericFlow", "pFlowHandle"]),
        # 
        'TcGetFlowNameA': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Byte"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FlowHandle", "StrSize", "pFlowName"]),
        # 
        'TcGetFlowNameW': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), label="LPArray", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FlowHandle", "StrSize", "pFlowName"]),
        # 
        'TcModifyFlow': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"SendingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "ReceivingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "TcObjectsLength": SimTypeInt(signed=False, label="UInt32"), "TcObjects": SimTypePointer(SimStruct({"ObjectType": SimTypeInt(signed=False, label="UInt32"), "ObjectLength": SimTypeInt(signed=False, label="UInt32")}, name="QOS_OBJECT_HDR", pack=False, align=None), offset=0)}, name="TC_GEN_FLOW", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FlowHandle", "pGenericFlow"]),
        # 
        'TcAddFilter': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimStruct({"AddressType": SimTypeShort(signed=False, label="UInt16"), "PatternSize": SimTypeInt(signed=False, label="UInt32"), "Pattern": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Mask": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="TC_GEN_FILTER", pack=False, align=None), offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FlowHandle", "pGenericFilter", "pFilterHandle"]),
        # 
        'TcDeregisterClient': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["ClientHandle"]),
        # 
        'TcDeleteFlow': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FlowHandle"]),
        # 
        'TcDeleteFilter': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["FilterHandle"]),
        # 
        'TcEnumerateFlows': SimTypeFunction([SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimStruct({"Length": SimTypeInt(signed=False, label="UInt32"), "OwnerProcessId": SimTypeInt(signed=False, label="UInt32"), "FlowNameLength": SimTypeShort(signed=False, label="UInt16"), "FlowName": SimTypeFixedSizeArray(SimTypeChar(label="Char"), 256), "pFlow": SimTypePointer(SimStruct({"SendingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "ReceivingFlowspec": SimStruct({"TokenRate": SimTypeInt(signed=False, label="UInt32"), "TokenBucketSize": SimTypeInt(signed=False, label="UInt32"), "PeakBandwidth": SimTypeInt(signed=False, label="UInt32"), "Latency": SimTypeInt(signed=False, label="UInt32"), "DelayVariation": SimTypeInt(signed=False, label="UInt32"), "ServiceType": SimTypeInt(signed=False, label="UInt32"), "MaxSduSize": SimTypeInt(signed=False, label="UInt32"), "MinimumPolicedSize": SimTypeInt(signed=False, label="UInt32")}, name="FLOWSPEC", pack=False, align=None), "TcObjectsLength": SimTypeInt(signed=False, label="UInt32"), "TcObjects": SimTypePointer(SimStruct({"ObjectType": SimTypeInt(signed=False, label="UInt32"), "ObjectLength": SimTypeInt(signed=False, label="UInt32")}, name="QOS_OBJECT_HDR", pack=False, align=None), offset=0)}, name="TC_GEN_FLOW", pack=False, align=None), offset=0), "NumberOfFilters": SimTypeInt(signed=False, label="UInt32"), "GenericFilter": SimTypePointer(SimStruct({"AddressType": SimTypeShort(signed=False, label="UInt16"), "PatternSize": SimTypeInt(signed=False, label="UInt32"), "Pattern": SimTypePointer(SimTypeBottom(label="Void"), offset=0), "Mask": SimTypePointer(SimTypeBottom(label="Void"), offset=0)}, name="TC_GEN_FILTER", pack=False, align=None), offset=0)}, name="ENUMERATION_BUFFER", pack=False, align=None), offset=0)], SimTypeInt(signed=False, label="UInt32"), arg_names=["IfcHandle", "pEnumHandle", "pFlowCount", "pBufSize", "Buffer"]),
    }

lib.set_prototypes(prototypes)



from . import SimHeapBase

class SimHeapLibc(SimHeapBase):
    """
    A class of heap that implements the major libc heap management functions.
    """

    def malloc(self, sim_size):
        """
        A somewhat faithful implementation of libc `malloc`.

        :param sim_size: the amount of memory (in bytes) to be allocated
        :returns:        the address of the allocation, or a NULL pointer if the allocation failed
        """
        raise NotImplementedError("%s not implemented for %s" % (self.malloc.__func__.__name__,
                                                                 self.__class__.__name__))

    def free(self, ptr):  #pylint:disable=unused-argument
        """
        A somewhat faithful implementation of libc `free`.

        :param ptr: the location in memory to be freed
        """
        raise NotImplementedError("%s not implemented for %s" % (self.free.__func__.__name__,
                                                                 self.__class__.__name__))

    def calloc(self, sim_nmemb, sim_size):
        """
        A somewhat faithful implementation of libc `calloc`.

        :param     sim_nmemb: the number of elements to allocated
        :param     sim_size: the size of each element (in bytes)
        :returns:  the address of the allocation, or a NULL pointer if the allocation failed
        """
        raise NotImplementedError("%s not implemented for %s" % (self.calloc.__func__.__name__,
                                                                 self.__class__.__name__))

    def realloc(self, ptr, size):
        """
        A somewhat faithful implementation of libc `realloc`.

        :param ptr:  the location in memory to be reallocated
        :param size: the new size desired for the allocation
        :returns:    the address of the allocation, or a NULL pointer if the allocation was freed or if no new allocation
                     was made
        """
        raise NotImplementedError("%s not implemented for %s" % (self.realloc.__func__.__name__,
                                                                 self.__class__.__name__))



from . import MemoryMixin

class UnwrapperMixin(MemoryMixin):
    """
    This mixin processes SimActionObjects by passing on their .ast field.
    """
    def store(self, addr, data, size=None, condition=None, **kwargs):
        return super().store(_raw_ast(addr), _raw_ast(data),
            size=_raw_ast(size),
            condition=_raw_ast(condition),
            **kwargs)

    def load(self, addr, size=None, condition=None, fallback=None, **kwargs):
        return super().load(_raw_ast(addr),
            size=_raw_ast(size),
            condition=_raw_ast(condition),
            fallback=_raw_ast(fallback),
            **kwargs)

    def find(self, addr, what, max_search, default=None, **kwargs):
        return super().find(_raw_ast(addr), _raw_ast(what), max_search,
            default=_raw_ast(default),
            **kwargs)

    def copy_contents(self, dst, src, size, condition=None, **kwargs):
        return super().copy_contents(_raw_ast(dst), _raw_ast(src), _raw_ast(size), _raw_ast(condition), **kwargs)

from ...state_plugins.sim_action_object import _raw_ast



# pylint:disable=line-too-long
import logging

from ...sim_type import SimTypeFunction,     SimTypeShort, SimTypeInt, SimTypeLong, SimTypeLongLong, SimTypeDouble, SimTypeFloat,     SimTypePointer,     SimTypeChar,     SimStruct,     SimTypeFixedSizeArray,     SimTypeBottom,     SimUnion,     SimTypeBool
from ...calling_conventions import SimCCStdcall, SimCCMicrosoftAMD64
from .. import SIM_PROCEDURES as P
from . import SimLibrary


_l = logging.getLogger(name=__name__)


lib = SimLibrary()
lib.set_default_cc('X86', SimCCStdcall)
lib.set_default_cc('AMD64', SimCCMicrosoftAMD64)
lib.set_library_names("hlink.dll")
prototypes = \
    {
        # 
        'HlinkCreateFromMoniker': SimTypeFunction([SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pimkTrgt", "pwzLocation", "pwzFriendlyName", "pihlsite", "dwSiteData", "piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkCreateFromString': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzTarget", "pwzLocation", "pwzFriendlyName", "pihlsite", "dwSiteData", "piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkCreateFromData': SimTypeFunction([SimTypeBottom(label="IDataObject"), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["piDataObj", "pihlsite", "dwSiteData", "piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkQueryCreateFromData': SimTypeFunction([SimTypeBottom(label="IDataObject")], SimTypeInt(signed=True, label="Int32"), arg_names=["piDataObj"]),
        # 
        'HlinkClone': SimTypeFunction([SimTypeBottom(label="IHlink"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pihl", "riid", "pihlsiteForClone", "dwSiteData", "ppvObj"]),
        # 
        'HlinkCreateBrowseContext': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkNavigateToStringReference': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IHlinkFrame"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IBindCtx"), SimTypeBottom(label="IBindStatusCallback"), SimTypeBottom(label="IHlinkBrowseContext")], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzTarget", "pwzLocation", "pihlsite", "dwSiteData", "pihlframe", "grfHLNF", "pibc", "pibsc", "pihlbc"]),
        # 
        'HlinkNavigate': SimTypeFunction([SimTypeBottom(label="IHlink"), SimTypeBottom(label="IHlinkFrame"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IBindCtx"), SimTypeBottom(label="IBindStatusCallback"), SimTypeBottom(label="IHlinkBrowseContext")], SimTypeInt(signed=True, label="Int32"), arg_names=["pihl", "pihlframe", "grfHLNF", "pbc", "pibsc", "pihlbc"]),
        # 
        'HlinkOnNavigate': SimTypeFunction([SimTypeBottom(label="IHlinkFrame"), SimTypeBottom(label="IHlinkBrowseContext"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pihlframe", "pihlbc", "grfHLNF", "pimkTarget", "pwzLocation", "pwzFriendlyName", "puHLID"]),
        # 
        'HlinkUpdateStackItem': SimTypeFunction([SimTypeBottom(label="IHlinkFrame"), SimTypeBottom(label="IHlinkBrowseContext"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pihlframe", "pihlbc", "uHLID", "pimkTrgt", "pwzLocation", "pwzFriendlyName"]),
        # 
        'HlinkOnRenameDocument': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IHlinkBrowseContext"), SimTypeBottom(label="IMoniker"), SimTypeBottom(label="IMoniker")], SimTypeInt(signed=True, label="Int32"), arg_names=["dwReserved", "pihlbc", "pimkOld", "pimkNew"]),
        # 
        'HlinkResolveMonikerForData': SimTypeFunction([SimTypeBottom(label="IMoniker"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IBindCtx"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"cfFormat": SimTypeShort(signed=False, label="UInt16"), "ptd": SimTypePointer(SimStruct({"tdSize": SimTypeInt(signed=False, label="UInt32"), "tdDriverNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdDeviceNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdPortNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdExtDevmodeOffset": SimTypeShort(signed=False, label="UInt16"), "tdData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="DVTARGETDEVICE", pack=False, align=None), offset=0), "dwAspect": SimTypeInt(signed=False, label="UInt32"), "lindex": SimTypeInt(signed=True, label="Int32"), "tymed": SimTypeInt(signed=False, label="UInt32")}, name="FORMATETC", pack=False, align=None), offset=0), SimTypeBottom(label="IBindStatusCallback"), SimTypeBottom(label="IMoniker")], SimTypeInt(signed=True, label="Int32"), arg_names=["pimkReference", "reserved", "pibc", "cFmtetc", "rgFmtetc", "pibsc", "pimkBase"]),
        # 
        'HlinkResolveStringForData': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IBindCtx"), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimStruct({"cfFormat": SimTypeShort(signed=False, label="UInt16"), "ptd": SimTypePointer(SimStruct({"tdSize": SimTypeInt(signed=False, label="UInt32"), "tdDriverNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdDeviceNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdPortNameOffset": SimTypeShort(signed=False, label="UInt16"), "tdExtDevmodeOffset": SimTypeShort(signed=False, label="UInt16"), "tdData": SimTypePointer(SimTypeChar(label="Byte"), offset=0)}, name="DVTARGETDEVICE", pack=False, align=None), offset=0), "dwAspect": SimTypeInt(signed=False, label="UInt32"), "lindex": SimTypeInt(signed=True, label="Int32"), "tymed": SimTypeInt(signed=False, label="UInt32")}, name="FORMATETC", pack=False, align=None), offset=0), SimTypeBottom(label="IBindStatusCallback"), SimTypeBottom(label="IMoniker")], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzReference", "reserved", "pibc", "cFmtetc", "rgFmtetc", "pibsc", "pimkBase"]),
        # 
        'HlinkParseDisplayName': SimTypeFunction([SimTypeBottom(label="IBindCtx"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=True, label="Int32"), SimTypePointer(SimTypeInt(signed=False, label="UInt32"), offset=0), SimTypePointer(SimTypeBottom(label="IMoniker"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pibc", "pwzDisplayName", "fNoForceAbs", "pcchEaten", "ppimk"]),
        # 
        'HlinkCreateExtensionServices': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeInt(signed=True, label="Int"), label="IntPtr", offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzAdditionalHeaders", "phwnd", "pszUsername", "pszPassword", "piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkPreprocessMoniker': SimTypeFunction([SimTypeBottom(label="IBindCtx"), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeBottom(label="IMoniker"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pibc", "pimkIn", "ppimkOut"]),
        # 
        'OleSaveToStreamEx': SimTypeFunction([SimTypeBottom(label="IUnknown"), SimTypeBottom(label="IStream"), SimTypeInt(signed=True, label="Int32")], SimTypeInt(signed=True, label="Int32"), arg_names=["piunk", "pistm", "fClearDirty"]),
        # 
        'HlinkSetSpecialReference': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["uReference", "pwzReference"]),
        # 
        'HlinkGetSpecialReference': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["uReference", "ppwzReference"]),
        # 
        'HlinkCreateShortcut': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IHlink"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["grfHLSHORTCUTF", "pihl", "pwzDir", "pwzFileName", "ppwzShortcutFile", "dwReserved"]),
        # 
        'HlinkCreateShortcutFromMoniker': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IMoniker"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["grfHLSHORTCUTF", "pimkTarget", "pwzLocation", "pwzDir", "pwzFileName", "ppwzShortcutFile", "dwReserved"]),
        # 
        'HlinkCreateShortcutFromString': SimTypeFunction([SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypeInt(signed=False, label="UInt32")], SimTypeInt(signed=True, label="Int32"), arg_names=["grfHLSHORTCUTF", "pwzTarget", "pwzLocation", "pwzDir", "pwzFileName", "ppwzShortcutFile", "dwReserved"]),
        # 
        'HlinkResolveShortcut': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeBottom(label="IHlinkSite"), SimTypeInt(signed=False, label="UInt32"), SimTypeBottom(label="IUnknown"), SimTypePointer(SimTypeBottom(label="Guid"), offset=0), SimTypePointer(SimTypePointer(SimTypeBottom(label="Void"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzShortcutFileName", "pihlsite", "dwSiteData", "piunkOuter", "riid", "ppvObj"]),
        # 
        'HlinkResolveShortcutToMoniker': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeBottom(label="IMoniker"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzShortcutFileName", "ppimkTarget", "ppwzLocation"]),
        # 
        'HlinkResolveShortcutToString': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzShortcutFileName", "ppwzTarget", "ppwzLocation"]),
        # 
        'HlinkIsShortcut': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzFileName"]),
        # 
        'HlinkGetValueFromParams': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzParams", "pwzName", "ppwzValue"]),
        # 
        'HlinkTranslateURL': SimTypeFunction([SimTypePointer(SimTypeChar(label="Char"), offset=0), SimTypeInt(signed=False, label="UInt32"), SimTypePointer(SimTypePointer(SimTypeChar(label="Char"), offset=0), offset=0)], SimTypeInt(signed=True, label="Int32"), arg_names=["pwzURL", "grfFlags", "ppwzTranslatedURL"]),
    }

lib.set_prototypes(prototypes)




import logging

from archinfo.arch_soot import SootMethodDescriptor

from .exceptions import SootMethodNotLoadedException

l = logging.getLogger('angr.engines.soot.method_dispatcher')


def resolve_method(state, method_name, class_name, params=(), ret_type=None,
                   include_superclasses=True, init_class=True,
                   raise_exception_if_not_found=False):
    """
    Resolves the method based on the given characteristics (name, class and
    params) The method may be defined in one of the superclasses of the given
    class (TODO: support interfaces).

    :rtype: archinfo.arch_soot.SootMethodDescriptor
    """
    base_class = state.javavm_classloader.get_class(class_name)
    if include_superclasses:
        class_hierarchy = state.javavm_classloader.get_class_hierarchy(base_class)
    else:
        class_hierarchy = [base_class]
    # walk up in class hierarchy, until method is found
    for class_descriptor in class_hierarchy:
        java_binary = state.project.loader.main_object
        soot_method = java_binary.get_soot_method(method_name, class_descriptor.name,
                                                  params, none_if_missing=True)
        if soot_method is not None:
            # init the class
            if init_class:
                state.javavm_classloader.init_class(class_descriptor)
            return SootMethodDescriptor.from_soot_method(soot_method)

    # method could not be found
    # => we are executing code that is not loaded (typically library code)
    # => fallback: continue with infos available from the invocation, so we
    #              still can use SimProcedures
    if raise_exception_if_not_found:
        raise SootMethodNotLoadedException()
    else:
        return SootMethodDescriptor(class_name, method_name, params, ret_type=ret_type)
